import os
import time
import random
import sys
import select

def play_game():
    width, height = 40, 20
    spaceship = [' ^ ', '/|\\', '/ \\']
    asteroid = ['***', '***', '***']
    score = 0
    lives = 3
    spaceship_x, spaceship_y = width // 2 - 1, height - 3

    asteroids = [(random.randint(0, width - 3), random.randint(0, height - 6)) for _ in range(5)]

    def clear_screen():
        os.system('cls' if os.name == 'nt' else 'clear')

    def draw(screen):
        clear_screen()
        for line in screen:
            print(''.join(line))
        print(f'Score: {score}  Lives: {lives}')
        print('Controls: A - Left, D - Right, Q - Quit')

    def move_asteroids():
        nonlocal score, lives
        new_asteroids = []
        for x, y in asteroids:
            if y < height - 3:
                new_asteroids.append((x, y + 1))
            else:
                score += 1
                new_asteroids.append((random.randint(0, width - 3), 0))
            # Check for collision
            if any(
                spaceship_x <= x + j < spaceship_x + 3 and spaceship_y <= y + i < spaceship_y + 3
                for i in range(3) for j in range(3)
            ):
                lives -= 1
                new_asteroids[-1] = (random.randint(0, width - 3), 0)
                if lives == 0:
                    print("Game Over!")
                    sys.exit()
        return new_asteroids

    def getch():
        if os.name == 'nt':
            import msvcrt
            if msvcrt.kbhit():
                return msvcrt.getch().decode('utf-8')
        else:
            import termios, tty
            fd = sys.stdin.fileno()
            old_settings = termios.tcgetattr(fd)
            try:
                tty.setraw(fd)
                if sys.stdin in select.select([sys.stdin], [], [], 0)[0]:
                    return sys.stdin.read(1)
            finally:
                termios.tcsetattr(fd, termios.TCSADRAIN, old_settings)
        return None

    def create_screen():
        return [[' ' for _ in range(width)] for _ in range(height)]

    def update_screen(screen):
        # Draw asteroids
        for x, y in asteroids:
            for i in range(3):
                for j in range(3):
                    if 0 <= y + i < height and 0 <= x + j < width:
                        screen[y + i][x + j] = '\033[91m' + asteroid[i][j] + '\033[0m'  # Red color for asteroids

        # Draw spaceship
        for i in range(3):
            for j in range(3):
                if 0 <= spaceship_y + i < height and 0 <= spaceship_x + j < width:
                    screen[spaceship_y + i][spaceship_x + j] = spaceship[i][j]

    screen = create_screen()
    clear_screen()

    while True:
        new_screen = create_screen()
        update_screen(new_screen)

        if screen != new_screen:
            draw(new_screen)
            screen = new_screen

        asteroids = move_asteroids()
        time.sleep(0.1)
        
        key = getch()
        if key:
            if key == 'q':
                break
            elif key == 'a' and spaceship_x > 0:
                spaceship_x -= 1
            elif key == 'd' and spaceship_x < width - 3:
                spaceship_x += 1

if __name__ == '__main__':
    play_game()



import cv2
import mediapipe as mp
import math

def open_camera(facial_keypoints=False):
    # Initialize mediapipe face mesh
    mp_face_detection = mp.solutions.face_detection
    mp_face_mesh = mp.solutions.face_mesh
    mp_drawing = mp.solutions.drawing_utils
    face_mesh = mp_face_mesh.FaceMesh(static_image_mode=False, max_num_faces=1, min_detection_confidence=0.5)
    face_detection = mp_face_detection.FaceDetection(min_detection_confidence=0.5)

    # Open the camera
    cap = cv2.VideoCapture(0)

    if not cap.isOpened():
        print("Error: Could not open video device.")
        return

    while True:
        # Capture frame-by-frame
        ret, frame = cap.read()

        if not ret:
            print("Error: Could not read frame.")
            break

        # Convert the BGR image to RGB
        rgb_frame = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)

        # Process the image and find faces
        face_detection_results = face_detection.process(rgb_frame)

        # Draw rectangle around the faces and detect facial keypoints if enabled
        if face_detection_results.detections:
            for detection in face_detection_results.detections:
                bboxC = detection.location_data.relative_bounding_box
                ih, iw, _ = frame.shape
                x, y, w, h = int(bboxC.xmin * iw), int(bboxC.ymin * ih), int(bboxC.width * iw), int(bboxC.height * ih)
                cv2.rectangle(frame, (x, y), (x + w, y + h), (0, 255, 0), 2)  # Green rectangle for detected face

                if facial_keypoints:
                    face_mesh_results = face_mesh.process(rgb_frame)
                    if face_mesh_results.multi_face_landmarks:
                        for face_landmarks in face_mesh_results.multi_face_landmarks:
                            mp_drawing.draw_landmarks(
                                image=frame,
                                landmark_list=face_landmarks,
                                connections=mp_face_mesh.FACEMESH_TESSELATION,
                                landmark_drawing_spec=mp_drawing.DrawingSpec(color=(0, 0, 255), thickness=1, circle_radius=1),
                                connection_drawing_spec=mp_drawing.DrawingSpec(color=(0, 255, 0), thickness=1, circle_radius=1)
                            )

                            # Estimate distance from camera based on face width
                            face_width_in_frame = w
                            # Constants for focal length and known face width (in centimeters)
                            focal_length = 500  # This is an example value, adjust it for your setup
                            known_face_width = 15  # Average human face width in cm
                            distance = (known_face_width * focal_length) / face_width_in_frame
                            cv2.putText(frame, f'Distance: {distance:.2f} cm', (x, y - 10), cv2.FONT_HERSHEY_SIMPLEX, 0.6, (255, 255, 255), 2)

        # Display the resulting frame
        cv2.imshow('Face Detection', frame)

        # Break the loop on 'q' key press
        if cv2.waitKey(1) & 0xFF == ord('q'):
            break

    # Release the capture and close windows
    cap.release()
    cv2.destroyAllWindows()



import json
import sys
import os
import cv2
import mediapipe as mp
from PySide6.QtWidgets import (
    QHBoxLayout, QApplication, QMainWindow, QLabel, QPushButton, QVBoxLayout, QWidget, QSlider, QComboBox
)
from PySide6.QtGui import QImage, QPixmap, QIcon
from PySide6.QtCore import QTimer, QDir, QUrl
from PySide6.QtMultimedia import QMediaPlayer, QAudioOutput
from cvtease.style import style
from .function.eyeAft import apply_glasses

def apply_app_icon(app, icon_path):
    icon = QIcon(icon_path)
    app.setWindowIcon(icon)

class FaceDetectionApp(QMainWindow):
    def __init__(self):
        super().__init__()
        self.setWindowTitle(style.WINDOW_TITLE)
        self.setGeometry(*style.WINDOW_GEOMETRY)
        self.setWindowIcon(QIcon("cvtease/asset/laugh.ico"))

        self.switch_button = QPushButton("Switch Glasses", self)
        self.switch_button.setStyleSheet(style.ROUND_BUTTON_STYLE)
        self.switch_button.clicked.connect(self.switch_glasses)
        
        self.face_mesh = mp.solutions.face_mesh.FaceMesh(
            max_num_faces=1, refine_landmarks=True,
            min_detection_confidence=0.5, min_tracking_confidence=0.5)
        self.mp_drawing = mp.solutions.drawing_utils
        self.mp_drawing_styles = mp.solutions.drawing_styles

        self.capture = cv2.VideoCapture(0)
        self.timer = QTimer()
        self.timer.timeout.connect(self.update_frame)
        self.timer.start(30)

        self.image_label = QLabel(self)
        self.image_label.setAlignment(style.IMAGE_LABEL_ALIGNMENT)

        # Create and style buttons
        self.start_button = QPushButton("Start Camera", self)
        self.start_button.setStyleSheet(style.ROUND_BUTTON_STYLE)
        self.start_button.clicked.connect(self.start_camera)

        self.capture_button = QPushButton("Capture Photo", self)
        self.capture_button.setStyleSheet(style.ROUND_BUTTON_STYLE)
        self.capture_button.clicked.connect(self.capture_photo)

        self.stop_button = QPushButton("Stop Camera", self)
        self.stop_button.setStyleSheet(style.ROUND_BUTTON_STYLE)
        self.stop_button.clicked.connect(self.stop_camera)

        # Create a horizontal layout for the buttons
        self.button_layout = QHBoxLayout()
        self.button_layout.addWidget(self.start_button)
        self.button_layout.addWidget(self.capture_button)
        self.button_layout.addWidget(self.stop_button)

        # Load the capture sound using QMediaPlayer
        self.media_player = QMediaPlayer()
        self.audio_output = QAudioOutput()
        self.media_player.setAudioOutput(self.audio_output)
        self.media_player.setSource(QUrl.fromLocalFile("cvtease/asset/photoCapture.wav"))

        # Settings Panel
        self.settings_panel = QWidget()
        self.settings_layout = QVBoxLayout()

        self.slider_detection = QSlider(style.SLIDER_TICK_POSITION)
        self.slider_detection.setRange(*style.SLIDER_RANGE)
        self.slider_detection.setValue(style.SLIDER_DEFAULT_VALUE)
        self.slider_detection.setTickInterval(style.SLIDER_TICK_INTERVAL)
        self.slider_detection.setTickPosition(QSlider.TicksBelow)
        self.slider_detection.setStyleSheet(style.SLIDER_STYLE)
        self.slider_detection.valueChanged.connect(self.update_detection_confidence)

        with open('cvtease/database/eyeAftjson.json', 'r') as file:
            data = json.load(file)
        self.glasses_paths = data['images']
        self.glasses_index = 0
        self.current_glasses_path = self.glasses_paths[self.glasses_index]

        self.combo_box_connections = QComboBox()
        self.combo_box_connections.addItems(style.COMBO_BOX_ITEMS)
        self.combo_box_connections.setStyleSheet(style.COMBO_BOX_STYLE)
        self.combo_box_connections.currentIndexChanged.connect(self.update_connections)

        self.settings_layout.addWidget(self.combo_box_connections)
        self.settings_panel.setLayout(self.settings_layout)

        self.label_detection = QLabel(f"Detection Confidence: 0.5")
        self.settings_layout.addWidget(self.label_detection)
        self.settings_layout.addWidget(self.slider_detection)

        # Under Development Label
        self.dev_label = QLabel(style.DEV_LABEL_STYLE, self)
        self.dev_label.setAlignment(style.IMAGE_LABEL_ALIGNMENT)
        self.dev_label.setStyleSheet(style.DEV_LABEL_STYLE)

        # Create and set the layout
        layout = QVBoxLayout()
        layout.addWidget(self.image_label)
        layout.addLayout(self.button_layout)  # Use the horizontal layout
        layout.addWidget(self.settings_panel)
        layout.addWidget(self.dev_label)
        layout.addWidget(self.switch_button)  # Add the switch button

        container = QWidget()
        container.setLayout(layout)
        self.setCentralWidget(container)

    def start_camera(self):
        self.capture.open(0)
        self.timer.start(30)

    def stop_camera(self):
        self.timer.stop()
        self.capture.release()
        self.image_label.clear()

    def update_frame(self):
        ret, frame = self.capture.read()
        if not ret:
            return

        frame_rgb = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
        results = self.face_mesh.process(frame_rgb)

        if results.multi_face_landmarks:
            for face_landmarks in results.multi_face_landmarks:
                self.draw_face_landmarks(frame, face_landmarks)

        q_img = QImage(frame.data, frame.shape[1], frame.shape[0], frame.strides[0], QImage.Format_RGB888)
        self.image_label.setPixmap(QPixmap.fromImage(q_img))

    def draw_face_landmarks(self, image, face_landmarks):
        connections = self.get_connections_from_combo()
        if connections == mp.solutions.face_mesh.FACEMESH_IRISES:
            apply_glasses(image, face_landmarks.landmark)
        else:
            self.mp_drawing.draw_landmarks(
                image=image,
                landmark_list=face_landmarks,
                connections=connections,
                landmark_drawing_spec=None,
                connection_drawing_spec=self.mp_drawing_styles
                .get_default_face_mesh_tesselation_style())

    def update_detection_confidence(self, value):
        confidence = value / 100.0
        self.face_mesh = mp.solutions.face_mesh.FaceMesh(
            max_num_faces=1, refine_landmarks=True,
            min_detection_confidence=confidence, min_tracking_confidence=0.5)
        self.label_detection.setText(f"Detection Confidence: {confidence}")

    def update_connections(self):
        self.update_frame()

    def get_connections_from_combo(self):
        index = self.combo_box_connections.currentIndex()
        if index == 0:
            return mp.solutions.face_mesh.FACEMESH_TESSELATION
        elif index == 1:
            return mp.solutions.face_mesh.FACEMESH_CONTOURS
        elif index == 2:
            return mp.solutions.face_mesh.FACEMESH_IRISES

    def capture_photo(self):
        ret, frame = self.capture.read()
        if not ret:
            print("Failed to capture frame")
            return

        frame_rgb = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)
        self.media_player.play()

        downloads_path = os.path.join(QDir.homePath(), "Downloads")
        if not os.path.exists(downloads_path):
            os.makedirs(downloads_path)
        photo_path = os.path.join(downloads_path, "captured_photo.jpg")

        cv2.imwrite(photo_path, frame_rgb)
        print(f"Photo captured and saved as {photo_path}")

    def switch_glasses(self):
        # Logic to switch between different PNGs
        self.glasses_index = (self.glasses_index + 1) % len(self.glasses_paths)
        self.current_glasses_path = self.glasses_paths[self.glasses_index]
        print(f"Switched to {self.current_glasses_path}")


if __name__ == "__main__":
    app = QApplication(sys.argv)
    
    # Apply the icon to the application
    apply_app_icon(app, "cvtease/asset/laugh.ico")
    
    window = FaceDetectionApp()
    window.show()
    
    sys.exit(app.exec())



from setuptools import setup, find_packages
from pathlib import Path

with open("README.md", encoding="utf-8") as f:
    long_description = f.read()
setup(
    name='cvtease',
    version='0.1.10',
    packages=find_packages(),
    install_requires=[
        'click',
        'colorama',
        'opencv-python',
        'mediapipe',
        'PySide6',  # Ensure compatibility with Python 3.7+
    ],
    entry_points={
        'console_scripts': [
            'cvtease=cvtease.cli:main',
        ],
    },
    author='w3cdpass',
    author_email='kupasva663@gmail.com',
    description='A CLI tool for computer vision that is under development. Includes a space shooter game using ASCII art.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/w3cdpass/cvtease',
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.7',
)
